import logging

from .provider import provider

__alias__ = 'QQ 音乐'
__feeluown_version__ = '1.1.0'
__version__ = '0.3a0'
__desc__ = 'QQ 音乐'

logger = logging.getLogger(__name__)


def enable(app):
    app.library.register(provider)
    if app.mode & app.GuiMode:
        from .provider_ui import ProviderUI

        provider_ui = ProviderUI(app)
        app.pvd_ui_mgr.register(provider_ui)


def disable(app):
    app.library.deregister(provider)
    if app.mode & app.GuiMode:
        app.providers.remove(provider.identifier)



import logging
from typing import List, Optional, Protocol
from feeluown.excs import ModelNotFound
from feeluown.library import (
    AbstractProvider,
    BriefSongModel,
    PlaylistModel,
    Collection,
    CollectionType,
    ProviderV2,
    ProviderFlags as PF,
    SupportsSongGet,
    SupportsSongMultiQuality,
    SupportsSongMV,
    VideoModel,
    LyricModel,
    SupportsCurrentUser,
    SupportsVideoGet,
    SupportsSongLyric,
    SupportsAlbumGet,
    SupportsAlbumSongsReader,
    SupportsArtistGet,
    SupportsPlaylistGet,
    SupportsPlaylistSongsReader,
    SupportsRecACollectionOfSongs,
    SimpleSearchResult,
    SearchType,
    ModelType,
)
from feeluown.media import Media, Quality
from feeluown.utils.reader import create_reader, SequentialReader
from .api import API


logger = logging.getLogger(__name__)
UNFETCHED_MEDIA = object()
SOURCE = "qqmusic"


class Supports(
    SupportsSongGet,
    SupportsSongMultiQuality,
    SupportsSongMV,
    SupportsCurrentUser,
    SupportsVideoGet,
    SupportsSongLyric,
    SupportsAlbumGet,
    SupportsArtistGet,
    SupportsPlaylistGet,
    SupportsPlaylistSongsReader,
    SupportsRecACollectionOfSongs,
    SupportsAlbumSongsReader,
    Protocol,
):
    pass


class QQProvider(AbstractProvider, ProviderV2):
    class meta:
        identifier = "qqmusic"
        name = "QQ 音乐"
        flags = {
            ModelType.song: PF.similar,
            ModelType.none: PF.current_user,
        }

    def __init__(self):
        super().__init__()
        self.api = API()

    def _(self) -> Supports:
        return self

    @property
    def identifier(self):
        return "qqmusic"

    @property
    def name(self):
        return "QQ 音乐"

    def use_model_v2(self, mtype):
        return mtype in (
            ModelType.song,
            ModelType.album,
            ModelType.artist,
            ModelType.playlist,
        )

    def song_get(self, identifier):
        data = self.api.song_detail(identifier)
        return _deserialize(data, QQSongSchema)

    def song_get_mv(self, song):
        mv_id = self._model_cache_get_or_fetch(song, "mv_id")
        if mv_id == 0:
            return None
        video = self.video_get(mv_id)
        # mv 的名字是空的
        video.title = song.title
        return video

    def song_get_lyric(self, song):
        mid = self._model_cache_get_or_fetch(song, "mid")
        content = self.api.get_lyric_by_songmid(mid)
        return LyricModel(identifier=mid, source=SOURCE, content=content)

    def video_get(self, identifier):
        data = self.api.get_mv(identifier)
        if not data:
            raise ModelNotFound(f"mv:{identifier} not found")
        fhd = hd = sd = ld = None
        for file in data["mp4"]:
            if not file["url"]:
                continue
            file_type = file["filetype"]
            url = file["freeflow_url"][0]
            if file_type == 40:
                fhd = url
            elif file_type == 30:
                hd = url
            elif file_type == 20:
                sd = url
            elif file_type == 10:
                ld = url
            elif file_type == 0:
                pass
            else:
                logger.warning("There exists another quality:%s mv.", str(file_type))
        q_url_mapping = dict(fhd=fhd, hd=hd, sd=sd, ld=ld)
        video = VideoModel(
            identifier=identifier,
            source=SOURCE,
            title="未知名字",
            artists=[],
            duration=1,
            cover="",
        )
        video.cache_set("q_url_mapping", q_url_mapping)
        return video

    def video_get_media(self, video, quality):
        q_media_mapping = self._model_cache_get_or_fetch(video, "q_url_mapping")
        return Media(q_media_mapping[quality.value])

    def video_list_quality(self, video):
        q_media_mapping = self._model_cache_get_or_fetch(video, "q_url_mapping")
        return [Quality.Video(k) for k, v in q_media_mapping.items() if v]

    def song_list_quality(self, song) -> List[Quality.Audio]:
        """List all possible qualities

        Please ensure all the qualities are valid. `song_get_media(song, quality)`
        must not return None with a valid quality.
        """
        return list(self._song_get_q_media_mapping(song))

    def song_get_media(self, song, quality: Quality.Audio) -> Optional[Media]:
        """Get song's media by a specified quality

        :return: when quality is invalid, return None
        """
        q_media_mapping = self._song_get_q_media_mapping(song)
        quality_suffix = self._model_cache_get_or_fetch(song, "quality_suffix")
        mid = self._model_cache_get_or_fetch(song, "mid")
        media_id = self._model_cache_get_or_fetch(song, "media_id")
        media = q_media_mapping.get(quality)
        if media is UNFETCHED_MEDIA:
            for q, t, b, s in quality_suffix:
                if quality == Quality.Audio(q):
                    url = self.api.get_song_url_v2(mid, media_id, t)
                    if url:
                        media = Media(url, bitrate=b, format=s)
                        q_media_mapping[quality] = media
                    else:
                        media = None
                        q_media_mapping[quality] = None
                    break
            else:
                media = None
        return media

    def _song_get_q_media_mapping(self, song):
        q_media_mapping, exists = song.cache_get("q_media_mapping")
        if exists is True:
            return q_media_mapping
        quality_suffix = self._model_cache_get_or_fetch(song, "quality_suffix")
        mid = self._model_cache_get_or_fetch(song, "mid")
        media_id = self._model_cache_get_or_fetch(song, "media_id")
        q_media_mapping = {}
        # 注：self.quality_suffix 这里可能会触发一次网络请求
        for idx, (q, t, b, s) in enumerate(quality_suffix):
            url = self.api.get_song_url_v2(mid, media_id, t)
            if url:
                q_media_mapping[Quality.Audio(q)] = Media(url, bitrate=b, format=s)
                # 一般来说，高品质有权限 低品质也会有权限，减少网络请求。
                # 这里把值设置为 UNFETCHED_MEDIA，作为一个标记。
                for i in range(idx + 1, len(quality_suffix)):
                    q_media_mapping[
                        Quality.Audio(quality_suffix[i][0])
                    ] = UNFETCHED_MEDIA
                break
        song.cache_set("q_media_mapping", q_media_mapping)
        return q_media_mapping

    def artist_get(self, identifier):
        data_mid = self.api.artist_songs(int(identifier), 1, 0)["singerMid"]
        data_artist = self.api.artist_detail(data_mid)
        artist = _deserialize(data_artist, QQArtistSchema)
        return artist

    def artist_create_songs_rd(self, artist):
        return create_g(
            self.api.artist_songs, int(artist.identifier), _ArtistSongSchema
        )

    def artist_create_albums_rd(self, artist):
        return create_g(
            self.api.artist_albums, int(artist.identifier), _BriefAlbumSchema
        )

    def album_get(self, identifier):
        data_album = self.api.album_detail(int(identifier))
        if data_album is None:
            raise ModelNotFound
        album = _deserialize(data_album, QQAlbumSchema)
        return album

    def album_create_songs_rd(self, album):
        album = self.album_get(album.identifier)
        return create_reader(album.songs)

    def user_get(self, identifier):
        data = self.api.user_detail(identifier)
        data["creator"]["fav_pid"] = data["mymusic"][0]["id"]
        # 假设使用微信登陆，从网页拿到 cookie，cookie 里面的 uin 是正确的，
        # 而这个接口返回的 uin 则可能是 0，因此手动重置一下。
        data["creator"]["uin"] = identifier
        return _deserialize(data, QQUserSchema)

    def playlist_get(self, identifier):
        data = self.api.playlist_detail(int(identifier), limit=1000)
        return _deserialize(data, QQPlaylistSchema)

    def playlist_create_songs_rd(self, playlist):
        songs = self._model_cache_get_or_fetch(playlist, "songs")
        return create_reader(songs)

    def __rec_hot_playlists(self):
        user = self.get_current_user()
        if user is None:
            return []
        # pids = self.api.get_recommend_playlists_ids()
        # rec_playlists = [QQPlaylistModel.get(pid) for pid in pids]
        playlists = self.api.recommend_playlists()
        for pl in playlists:
            pl["dissid"] = pl["content_id"]
            pl["dissname"] = pl["title"]
            pl["logo"] = pl["cover"]
        return [_deserialize(playlist, QQPlaylistSchema) for playlist in playlists]

    def rec_list_daily_playlists(self):
        # TODO: cache API result
        feed = self.api.get_recommend_feed()
        shelf = None
        for shelf_ in feed['v_shelf']:
            # I guess 10046 means 'song'.
            if shelf_['extra_info'].get('moduleID', '').startswith('playlist'):
                shelf = shelf_
                break
        if shelf is None:
            return []
        playlists = []
        for batch in shelf['v_niche']:
            for card in batch['v_card']:
                print(card['title'], card['jumptype'])
                if card['jumptype'] == 10014:  # 10014->playlist
                    playlists.append(
                        PlaylistModel(identifier=str(card['id']),
                                      source=SOURCE,
                                      name=card['title'],
                                      cover=card['cover'],
                                      description=card['miscellany']['rcmdtemplate'],
                                      play_count=card['cnt'])
                    )
        return playlists

    def rec_a_collection_of_songs(self):
        # TODO: cache API result
        feed = self.api.get_recommend_feed()
        shelf = None
        for shelf_ in feed['v_shelf']:
            # I guess 10046 means 'song'.
            if int(shelf_['miscellany'].get('jumptype', 0)) == 10046:
                shelf = shelf_
                break
        if shelf is None:
            return Collection(name='',
                              type_=CollectionType.only_songs,
                              models=[],
                              description='')
        title = shelf['title_content'] or shelf['title_template']
        song_ids = []
        for batch in shelf['v_niche']:
            for card in batch['v_card']:
                if card['jumptype'] == 10046:
                    song_id = int(card['id'])
                    if song_id not in song_ids:
                        song_ids.append(song_id)

        tracks = self.api.batch_song_details(song_ids)
        return Collection(name=title,
                          type_=CollectionType.only_songs,
                          models=[_deserialize(track, QQSongSchema) for track in tracks],
                          description='')

    def current_user_get_radio_songs(self):
        songs_data = self.api.get_radio_music()
        return [_deserialize(s, QQSongSchema) for s in songs_data]

    def current_user_list_playlists(self):
        user = self.get_current_user()
        if user is None:
            return []
        playlists = self._model_cache_get_or_fetch(user, "playlists")
        return playlists

    def current_user_fav_create_songs_rd(self):
        user = self.get_current_user()
        if user is None:
            return create_reader([])
        fav_pid = self._model_cache_get_or_fetch(user, "fav_pid")
        playlist = self.playlist_get(fav_pid)
        reader = create_reader(self.playlist_create_songs_rd(playlist))
        return reader.readall()

    def current_user_fav_create_albums_rd(self):
        user = self.get_current_user()
        if user is None:
            return create_reader([])
        # TODO: fetch more if total count > 100
        albums = self.api.user_favorite_albums(user.identifier)
        return [_deserialize(album, _UserAlbumSchema) for album in albums]

    def current_user_fav_create_artists_rd(self):
        user = self.get_current_user()
        if user is None:
            return create_reader([])
        # TODO: fetch more if total count > 100
        mid = self._model_cache_get_or_fetch(user, "mid")
        artists = self.api.user_favorite_artists(user.identifier, mid)
        return [_deserialize(artist, _UserArtistSchema) for artist in artists]

    def current_user_fav_create_playlists_rd(self):
        user = self.get_current_user()
        if user is None:
            return create_reader([])
        mid = self._model_cache_get_or_fetch(user, "mid")
        playlists = self.api.user_favorite_playlists(user.identifier, mid)
        return [_deserialize(playlist, QQPlaylistSchema) for playlist in playlists]

    def has_current_user(self):
        return self._user is not None

    def get_current_user(self):
        return self._user

    def song_list_similar(self, song):
        data_songs = self.api.song_similar(int(song.identifier))
        return [_deserialize(data_song, QQSongSchema) for data_song in data_songs]


def _deserialize(data, schema_cls):
    schema = schema_cls()
    obj = schema.load(data)
    return obj


def create_g(func, identifier, schema):
    data = func(identifier, page=1)
    total = int(data["totalNum"] if schema == _ArtistSongSchema else data["total"])

    def g():
        nonlocal data
        if data is None:
            yield from ()
        else:
            page = 1
            while data["songList"] if schema == _ArtistSongSchema else data["list"]:
                obj_data_list = (
                    data["songList"] if schema == _ArtistSongSchema else data["list"]
                )
                for obj_data in obj_data_list:
                    obj = _deserialize(obj_data, schema)
                    yield obj
                page += 1
                data = func(identifier, page)

    return SequentialReader(g(), total)


def search(keyword, **kwargs):
    type_ = SearchType.parse(kwargs["type_"])
    type_type_map = {
        SearchType.so: 0,
        SearchType.ar: 1,
        SearchType.al: 2,
        SearchType.pl: 3,
        SearchType.vi: 4,
    }
    data = provider.api.search(keyword, type_=type_type_map[type_])
    if type_ == SearchType.so:
        songs = [_deserialize(song, QQSongSchema) for song in data]
        return SimpleSearchResult(q=keyword, songs=songs)
    if type_ == SearchType.ar:
        artists = [_deserialize(artist, SearchArtistSchema) for artist in data]
        return SimpleSearchResult(q=keyword, artists=artists)
    elif type_ == SearchType.al:
        albums = [_deserialize(album, SearchAlbumSchema) for album in data]
        return SimpleSearchResult(q=keyword, albums=albums)
    elif type_ == SearchType.pl:
        playlists = [_deserialize(playlist, SearchPlaylistSchema) for playlist in data]
        return SimpleSearchResult(q=keyword, playlists=playlists)
    elif type_ == SearchType.vi:
        models = [_deserialize(model, SearchMVSchema) for model in data]
        return SimpleSearchResult(q=keyword, videos=models)


provider = QQProvider()
provider.search = search


from .schemas import (  # noqa
    QQSongSchema,
    QQArtistSchema,
    _ArtistSongSchema,
    _BriefAlbumSchema,
    _UserArtistSchema,
    _BriefArtistSchema,
    QQAlbumSchema,
    QQPlaylistSchema,
    QQUserSchema,
    _UserAlbumSchema,
    SearchAlbumSchema,
    SearchArtistSchema,
    SearchPlaylistSchema,
    SearchMVSchema,
)  # noqa



import logging

from marshmallow import Schema, fields, post_load, EXCLUDE
from feeluown.library import (
    SongModel,
    BriefAlbumModel,
    BriefArtistModel,
    ArtistModel,
    AlbumModel,
    PlaylistModel,
    BriefPlaylistModel,
    UserModel,
    VideoModel,
)

logger = logging.getLogger(__name__)


class BaseSchema(Schema):
    source = fields.Str(missing="qqmusic")

    class Meta:
        unknown = EXCLUDE


SOURCE = "qqmusic"
Schema = BaseSchema


def get_cover(mid, type_):
    """获取专辑、歌手封面

    :param type_: 专辑： 2，歌手：1
    """
    return f'http://y.gtimg.cn/music/photo_new/T00{type_}R800x800M000{mid}.jpg'


def create_model(model_cls, data, fields_to_cache=None):
    """
    maybe this function should be provided by feeluown

    :param fields_to_cache: list of fields name to be cached
    """
    if fields_to_cache is not None:
        cache_data = {}
        for field in fields_to_cache:
            value = data.pop(field)
            if value is not None:
                cache_data[field] = value
        model = model_cls(**data)
        for field, value in cache_data.items():
            model.cache_set(field, value)
    else:
        model = model_cls(**data)
    return model


def pop_album_from_data(data):
    album_id = data.pop("albumid")
    album_mid = data.pop("albummid")
    album_name = data.pop("albumname")
    album_data = {
        "identifier": album_id,
        "source": SOURCE,
        "mid": album_mid,
        "name": album_name,
    }
    return create_model(BriefAlbumModel, album_data, ["mid"])


class _SongArtistSchema(Schema):
    identifier = fields.Int(data_key="id", required=True)
    name = fields.Str(data_key="name", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return BriefArtistModel(**data)


class _SongAlbumSchema(Schema):
    identifier = fields.Int(data_key="id", required=True)
    name = fields.Str(data_key="name", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return BriefAlbumModel(**data)


class QQSongSchema(Schema):
    identifier = fields.Int(data_key="id", required=True)
    mid = fields.Str(data_key="mid", required=True)
    duration = fields.Float(data_key="interval", required=True)
    title = fields.Str(data_key="title", required=True)
    artists = fields.List(fields.Nested("_SongArtistSchema"), data_key="singer")
    album = fields.Nested("_SongAlbumSchema", required=True)
    files = fields.Dict(data_key="file", missing={})
    mv = fields.Dict(required=True)

    @post_load
    def create_model(self, data, **kwargs):
        song = SongModel(
            identifier=data["identifier"],
            source=SOURCE,
            duration=data["duration"] * 1000,
            title=data["title"],
            artists=data.get("artists", []),
            album=data["album"],
        )
        song.cache_set("mid", data["mid"])
        song.cache_set("media_id", data["files"]["media_mid"])
        song.cache_set("mv_id", data["mv"].get("vid", 0))
        # 记录有哪些资源文件, 没有权限的用户依然获取不到
        quality_suffix = []
        files = data["files"]
        if files.get("size_flac"):  # has key and value is not empty
            quality_suffix.append(("shq", "F000", 800, "flac"))
        elif files.get("size_ape"):
            quality_suffix.append(("shq", "A000", 800, "ape"))
        if files.get("size_320") or files.get("size_320mp3"):
            quality_suffix.append(("hq", "M800", 320, "ape"))
        if files.get("size_aac") or files.get("size_192aac"):
            quality_suffix.append(("sq", "C600", 192, "m4a"))
        if files.get("size_128") or files.get("size_128mp3"):
            quality_suffix.append(("lq", "M500", 128, "mp3"))
        song.cache_set("quality_suffix", quality_suffix)
        return song


class _ArtistSongSchema(Schema):
    value = fields.Nested(QQSongSchema, data_key="songInfo")

    @post_load
    def create_model(self, data, **kwargs):
        return data["value"]


class _BriefArtistSchema(Schema):
    identifier = fields.Int(data_key="singerID", required=True)
    mid = fields.Str(data_key="singerMID", required=True)
    name = fields.Str(data_key="singerName", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return create_model(BriefArtistModel, data, ["mid"])


class SearchArtistSchema(_BriefArtistSchema):
    pic_url = fields.Str(data_key="singerPic", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        data['hot_songs'] = []
        data['description'] = ''
        data['aliases'] = []
        return create_model(ArtistModel, data, ["mid"])


class _BriefAlbumSchema(Schema):
    identifier = fields.Int(data_key="albumID", required=True)
    mid = fields.Str(data_key="albumMID", required=True)
    name = fields.Str(data_key="albumName", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return create_model(BriefAlbumModel, data, ["mid"])


class SearchAlbumSchema(_BriefAlbumSchema):
    cover = fields.Str(data_key="albumPic", required=True)
    released = fields.Str(data_key="publicTime", required=True)
    song_count = fields.Int(required=True)
    artists = fields.List(fields.Nested(_SongArtistSchema),
                          data_key="singer_list",
                          required=True)
    @post_load
    def create_model(self, data, **kwargs):
        data['description'] = ''
        data['songs'] = []
        return create_model(AlbumModel, data, ['mid'])


class _BriefPlaylistSchema(Schema):
    identifier = fields.Int(data_key="dissid", required=True)
    name = fields.Str(data_key="dissname", required=True)
    cover = fields.Str(data_key="imgurl", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        data['description'] = ''
        return create_model(PlaylistModel, **data)


class PlaylistUserSchema(Schema):
    identifier = fields.Str(data_key="creator_uin", required=True)
    mid = fields.Str(data_key="encrypt_uin", required=True)
    name = fields.Str(required=True)
    avatar_url = fields.Str(required=True, data_key="avatarUrl")

    @post_load
    def create_model(self, data, **kwargs):
        return create_model(UserModel, data, ['mid'])


class SearchPlaylistSchema(_BriefPlaylistSchema):
    creator = fields.Nested("PlaylistUserSchema", required=True)
    description = fields.Str(data_key="introduction", required=True)
    play_count = fields.Int(data_key="listennum", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return create_model(PlaylistModel, data)


class QQArtistSchema(Schema):
    """歌手详情 Schema、歌曲歌手简要信息 Schema"""

    identifier = fields.Int(data_key="singer_id", required=True)
    mid = fields.Str(data_key="singer_mid", required=True)
    name = fields.Str(data_key="singer_name", required=True)

    description = fields.Str(data_key="SingerDesc", missing="")
    hot_songs = fields.List(
        fields.Nested(_ArtistSongSchema), data_key="list", missing=list
    )

    @post_load
    def create_model(self, data, **kwargs):
        data["pic_url"] = get_cover(data['mid'], 1)
        data["aliases"] = []
        return create_model(ArtistModel, data, ["mid"])


class QQAlbumSchema(Schema):
    album_info = fields.Dict(data_key="getAlbumInfo", required=True)
    album_desc = fields.Dict(data_key="getAlbumDesc", required=True)
    # 非中文专辑会把专辑的中文翻译加进去, 为保持前后一致此外去掉翻译文字
    artist_info = fields.Dict(data_key="getSingerInfo", required=True)

    # 有的专辑歌曲列表为 null，比如：fuo://qqmusic/albums/8623
    songs = fields.List(
        fields.Nested(QQSongSchema), data_key="getSongInfo", allow_none=True
    )

    @post_load
    def create_model(self, data, **kwargs):
        singer_name = data["artist_info"]["Fsinger_name"]
        artist = BriefArtistModel(
            identifier=data["artist_info"]["Fsinger_id"],
            source=SOURCE,
            # split('/')：有的专辑有个多歌手，只有第一个才是正确的专辑艺人
            # split('(')：有的非中文歌手拥有别名在括号里
            name=singer_name.split("/")[0].split("(")[0].strip(),
        )
        # 非中文专辑会把专辑的中文翻译加进去, 为保持前后一致此外去掉括号里的中文翻译
        if data["songs"]:
            album_name = data["songs"][0].album.name
        else:
            album_name = data["album_info"]["Falbum_name"]
        mid = data["album_info"]["Falbum_mid"]
        album = AlbumModel(
            identifier=data["album_info"]["Falbum_id"],
            source=SOURCE,
            name=album_name,
            description=data["album_desc"]["Falbum_desc"],
            songs=data["songs"] or [],
            artists=[artist],
            cover=get_cover(mid, 2)
        )
        album.cache_set("mid", mid)
        return album


class QQPlaylistSchema(Schema):
    identifier = fields.Int(required=True, data_key="dissid")
    name = fields.Str(required=True, data_key="dissname")
    cover = fields.Str(required=True, data_key="logo")
    # songs field maybe null, though it can't be null in model
    songs = fields.List(
        fields.Nested(QQSongSchema), data_key="songlist", allow_none=True
    )

    @post_load
    def create_model(self, data, **kwargs):
        fields_to_cache = []
        if data.get('songs') is not None:
            fields_to_cache = ['songs']
        data['description'] = ''
        return create_model(PlaylistModel, data, fields_to_cache)


class _UserArtistSchema(Schema):
    other_infos = fields.Dict(data_key="OtherInfo", required=True)
    mid = fields.Str(data_key="MID", required=True)
    name = fields.Str(data_key="Name", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        data["identifier"] = data.pop("other_infos")["SingerID"]
        return create_model(BriefArtistModel, data, ['mid'])


class _UserAlbumSchema(Schema):
    identifier = fields.Int(data_key="albumid", required=True)
    mid = fields.Str(data_key="albummid", required=True)
    name = fields.Str(data_key="albumname", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return create_model(BriefAlbumModel, data, ["mid"])


class _UserPlaylistSchema(Schema):
    identifier = fields.Int(data_key="dissid", required=True)
    name = fields.Str(data_key="title", required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return create_model(BriefPlaylistModel, data)


class QQUserSchema(Schema):
    creator = fields.Dict(required=True)
    mydiss = fields.Dict(required=True)

    @post_load
    def create_model(self, data, **kwargs):
        creator = data["creator"]
        playlists_data = data["mydiss"]["list"]
        schema = _UserPlaylistSchema()
        playlists = []
        for each in playlists_data:
            playlist = schema.load(each)
            playlists.append(playlist)
        data = dict(
            identifier=creator["uin"],
            source=SOURCE,
            mid=creator["encrypt_uin"],
            name=creator["nick"],
            fav_pid=creator["fav_pid"],
            avatar_url=creator["headpic"],
            playlists=playlists,
        )
        return create_model(UserModel, data, ['mid', 'fav_pid', 'playlists'])


class SearchMVSchema(Schema):
    # 使用 mv_id 字段的话，目前拿不到播放 url，用 v_id  比较合适
    identifier = fields.Str(data_key="v_id", required=True)
    title = fields.Str(data_key="mv_name", required=True)
    artists = fields.List(fields.Nested("_SongArtistSchema"),
                          data_key="singer_list",
                          required=True)
    duration = fields.Int(required=True)
    cover = fields.Str(data_key="mv_pic_url", required=True)
    play_count = fields.Int(required=True)

    @post_load
    def create_model(self, data, **kwargs):
        return create_model(VideoModel, data)



import json
import logging
import os

from feeluown.utils.dispatch import Signal
from feeluown.utils.aio import run_fn
from feeluown.consts import DATA_DIR
from feeluown.gui.widgets.login import CookiesLoginDialog, InvalidCookies
from feeluown.gui.provider_ui import AbstractProviderUi
from feeluown.app.gui_app import GuiApp

from .provider import provider
from .excs import QQIOError

logger = logging.getLogger(__name__)


USER_INFO_FILE = DATA_DIR + '/qqmusic_user_info.json'


def read_cookies():
    if os.path.exists(USER_INFO_FILE):
        # if the file is broken, just raise error
        with open(USER_INFO_FILE) as f:
            return json.load(f).get('cookies', None)


class ProviderUI(AbstractProviderUi):
    def __init__(self, app: GuiApp):
        self._app = app
        self._login_event = Signal()

    @property
    def provider(self):
        return provider

    def get_colorful_svg(self) -> str:
        return os.path.join(os.path.dirname(__file__), 'assets', 'icon.svg')

    def login_or_go_home(self):
        if provider._user is None:
            # According to #14, we have two ways to login:
            # 1. the default way, as the code shows
            # 2. a way for VIP user(maybe):
            #    - url: https://xui.ptlogin2.qq.com/cgi-bin/xlogin?appid=1006102
            #           &daid=384&low_login=1&pt_no_auth=1
            #           &s_url=https://y.qq.com/vip/daren_recruit/apply.html&style=40
            #
            #    - keys: ['skey']
            url = os.getenv('FUO_QQMUSIC_LOGIN_URL', 'https://y.qq.com')
            keys = os.getenv('FUO_QQMUSIC_LOGIN_COOKIE_KEYS', 'qqmusic_key').split(',')
            self._dialog = LoginDialog(url, keys)
            self._dialog.login_succeed.connect(self.on_login_succeed)
            self._dialog.show()
            self._dialog.autologin()
        else:
            logger.info('already logged in')
            self.login_event.emit(self, 2)

    @property
    def login_event(self):
        return self._login_event

    def on_login_succeed(self):
        del self._dialog
        self.login_event.emit(self, 1)


class LoginDialog(CookiesLoginDialog):

    def setup_user(self, user):
        provider._user = user

    async def user_from_cookies(self, cookies):
        if not cookies:  # is None or empty
            raise InvalidCookies('empty cookies')

        uin = provider.api.get_uin_from_cookies(cookies)
        if uin is None:
            raise InvalidCookies("can't extract user info from cookies")

        provider.api.set_cookies(cookies)
        # try to extract current user
        try:
            user = await run_fn(provider.user_get, uin)
        except QQIOError:
            provider.api.set_cookies(None)
            raise InvalidCookies('get user info with cookies failed, expired cookies?')
        else:
            return user

    def load_user_cookies(self):
        return read_cookies()

    def dump_user_cookies(self, user, cookies):
        js = {
            'identifier': user.identifier,
            'name': user.name,
            'cookies': cookies
        }
        with open(USER_INFO_FILE, 'w') as f:
            json.dump(js, f, indent=2)
